@@ -5,11 +5,12 @@ from requests.auth import HTTPBasicAuth
 
 from qubell.api.private.exceptions import ApiUnauthorizedError
 from qubell.api.provider import route, play_auth, basic_auth
+from qubell.api.globals import QUBELL as qubell_config
 
 
 class Router(object):
-    def __init__(self, base_url, verify_ssl=False, verify_codes=True):
-        self.base_url = base_url
+    def __init__(self, base_url=None, verify_ssl=False, verify_codes=True):
+        self.base_url = base_url or qubell_config['tenant']
         self.verify_ssl = verify_ssl
         self.verify_codes = verify_codes
 
@@ -17,17 +18,23 @@ class Router(object):
         self._auth = None
         self.public_api_in_use = False
 
+        self._creds = None
+
+        self._session = requests.Session()
+
     @property
     def is_connected(self):
         return self._cookies and 'PLAY_SESSION' in self._cookies
 
-    #todo: add integration test for this
-    def connect(self, email, password):
+    def connect(self, email=None, password=None):
+        email = email or qubell_config['user']
+        password = password or qubell_config['password']
         url = self.base_url + '/signIn'
         data = {
             'email': email,
             'password': password}
-        with requests.session() as session:
+
+        with self._session as session:
             session.post(url=url, data=data, verify=self.verify_ssl)
             self._cookies = session.cookies
 
@@ -36,6 +43,20 @@ class Router(object):
 
         self._auth = HTTPBasicAuth(email, password)
 
+        self._creds = email, password
+
+
+class InstanceRouter(object):
+    """
+    Router dependency
+    """
+    _router = None
+    def init_router(self, router):
+        assert router, "router cannot be None"
+        self._router = router
+        return self
+
+
 class PrivatePath(Router):
 
     @route("POST /signIn")
@@ -98,6 +119,10 @@ class PrivatePath(Router):
     @route("POST /organizations/{org_id}/applications/{app_id}/manifests{ctype}")
     def post_application_manifest(self, org_id, app_id, data, files, cookies, ctype=".json"): pass
 
+    @play_auth
+    @route("GET /organizations/{org_id}/applications/{app_id}/manifests/latest{ctype}")
+    def get_application_manifests_latest(self, org_id, app_id, cookies, ctype=".json"): pass
+
     #Revision
     @play_auth
     @route("POST /organizations/{org_id}/applications/{app_id}/createRevision{ctype}")
@@ -111,14 +136,22 @@ class PrivatePath(Router):
     @route("GET /organizations/{org_id}/applications/{app_id}/revisions/{rev_id}{ctype}")
     def get_revision(self, org_id, app_id, rev_id, cookies, ctype=".json"): pass
 
+    @play_auth
+    @route("GET /organizations/{org_id}/applications/{app_id}/revisions{ctype}")
+    def get_revisions(self, org_id, app_id, cookies, ctype=".json"): pass
+
     @play_auth
     @route("DELETE /organizations/{org_id}/applications/{app_id}/revisions/{rev_id}{ctype}")
     def delete_revision(self, org_id, app_id, rev_id, cookies, data="{}", ctype=".json"): pass
 
+    @play_auth
+    @route("DELETE /organizations/{org_id}/applications/{app_id}/destroyedInstances{ctype}")
+    def delete_destroyed_instances(self, org_id, app_id, cookies, data="{}", ctype=".json"): pass
+
     #Instance
     @play_auth
     @route("GET /organizations/{org_id}/dashboard{ctype}")
-    def get_instances(self, org_id, cookies, ctype=".json"): pass
+    def get_instances(self, org_id, cookies, params=None, ctype=".json"): pass
 
     @play_auth
     @route("GET /organizations/{org_id}/instances/{instance_id}{ctype}")
@@ -128,10 +161,22 @@ class PrivatePath(Router):
     @route("POST /organizations/{org_id}/instances/{instance_id}/workflows/{wf_name}{ctype}")
     def post_instance_workflow(self, org_id, instance_id, wf_name, cookies, data="{}", ctype=".json"): pass
 
+    @play_auth
+    @route("POST /organizations/{org_id}/instances/{instance_id}/components/{component_path}/workflows/{wf_name}{ctype}")
+    def post_instance_component_workflow(self, org_id, instance_id, component_path, wf_name, cookies, data="{}", ctype=".json"): pass
+
     @play_auth
     @route("PUT /organizations/{org_id}/instances/{instance_id}/configuration{ctype}")
     def put_instance_configuration(self, org_id, instance_id, data, cookies, ctype=".json"): pass
 
+    @play_auth
+    @route("GET /organizations/{org_id}/instances/{instance_id}/configuration{ctype}")
+    def get_instance_configuration(self, org_id, instance_id, cookies, ctype=".json"): pass
+
+    @play_auth
+    @route("PUT /organizations/{org_id}/instances/{instance_id}/rename{ctype}")
+    def put_instance_rename(self, org_id, instance_id, data, cookies, ctype=".json"): pass
+
     @play_auth
     @route("POST /organizations/{org_id}/environments/updateServiceEnvs/{instance_id}{ctype}")
     def post_instance_services(self, org_id, instance_id, data, cookies, ctype=".json"): pass
@@ -142,11 +187,7 @@ class PrivatePath(Router):
 
     @play_auth
     @route("GET /organizations/{org_id}/instances/{instance_id}/activitylog{ctype}")
-    def get_instance_activitylog(self, org_id, instance_id, cookies, ctype=".json"): pass
-
-    @play_auth
-    @route("GET /organizations/{org_id}/instances/{instance_id}/activitylog{ctype}?after={timestamp}")
-    def get_instance_activitylog_after(self, org_id, instance_id, timestamp, cookies, ctype=".json"): pass
+    def get_instance_activitylog(self, org_id, instance_id, cookies, params=None, ctype=".json"): pass
 
     @play_auth
     @route("POST /organizations/{org_id}/instances/{instance_id}/{action}{ctype}")
@@ -164,6 +205,14 @@ class PrivatePath(Router):
     @route("POST /organizations/{org_id}/instances/{instance_id}/storedWorkflows/{workflow_id}/reschedule{ctype}")
     def post_instance_reschedule(self, org_id, instance_id, workflow_id, data, cookies, ctype=".json"): pass
 
+    @play_auth
+    @route("GET /organizations/{org_id}/runtime-components/{component_id}{ctype}")
+    def get_component_details(self, org_id, component_id, cookies, ctype=".json"): pass
+
+    @play_auth
+    @route("GET /organizations/{org_id}/runtime-components{ctype}")
+    def get_components(self, org_id, cookies, params=None, ctype=".json"): pass
+
     #Environment
     @play_auth
     @route("GET /organizations/{org_id}/environments{ctype}")
@@ -185,6 +234,10 @@ class PrivatePath(Router):
     @route("DELETE /organizations/{org_id}/environments/{env_id}{ctype}")
     def delete_environment(self, org_id, env_id, cookies, data="{}", ctype=".json"): pass
 
+    @play_auth
+    @route("POST /organizations/{org_id}/environments/{env_id}/import{ctype}")
+    def post_env_import(self, org_id, env_id, cookies, data="{}", files="{}", ctype=".json"): pass
+
     #Zone
     @play_auth
     @route("GET /organizations/{org_id}/zones{ctype}")
@@ -199,6 +252,18 @@ class PrivatePath(Router):
     @route("POST /organizations/{org_id}/services/{instance_id}/keys/generate{ctype}")
     def post_service_generate(self, org_id, instance_id, cookies, data="{}", ctype=".json"): pass
 
+    @play_auth
+    @route("GET /organizations/{org_id}/services/{instance_id}/keys{ctype}")
+    def get_service_keys(self, org_id, instance_id, cookies, ctype=".json"): pass
+
+    @play_auth
+    @route("GET /organizations/{org_id}/services/{instance_id}/keys/{key_id}/id_rsa.pub")
+    def get_service_public_key(self, org_id, instance_id, key_id, cookies, ctype=".json"): pass
+
+    @play_auth
+    @route("GET /organizations/{org_id}/environments/{env_id}/id_rsa")
+    def get_environment_default_private_key(self, org_id, env_id, cookies): pass
+
     # Role
     @play_auth
     @route("POST /organizations/{org_id}/roles{ctype}")
@@ -238,9 +303,50 @@ class PrivatePath(Router):
     def evict_user(self, org_id, cookies, user_id, data="{}", ctype=".json"): pass
 
     @play_auth
-    @route("POST /organizations/invite{ctype}")
+    @route("POST /invite{ctype}")
     def invite_user(self, cookies, data="{}", ctype=".json"): pass
 
+    @route("POST /quickSignUp")
+    def post_quick_sign_up(self, data=None, params=None, files=None): pass
+
+    @play_auth
+    @route("POST /organizations/{org_id}/init.json")
+    def post_init(self, org_id, data, cookies): pass
+
+    @play_auth
+    @route("POST /organizations/{org_id}/initCustomCloudAccount.json")
+    def post_init_custom_cloud_account(self, org_id, data, cookies): pass
+
+    @play_auth
+    @route("GET /organizations/{org_id}/welcomeWizardComponents.json")
+    def get_welcome_wizard_components(self, org_id, cookies): pass
+
+    @play_auth
+    @route("POST /organizations/{org_id}/initDockerService.json")
+    def post_init_docker_service(self, org_id, cookies, data="{}"): pass
+
+    @play_auth
+    @route("GET /applications/upload{ctype}")
+    def get_upload(self, params, cookies, ctype=".json"): pass
+
+    @play_auth
+    @route("GET /organizations/{org_id}/categories{ctype}")
+    def get_categories(self, org_id, cookies, ctype=".json"): pass
+
+    @play_auth
+    @route("POST /organizations/{org_id}/application-kits.json")
+    def post_application_kits(self, org_id, data, cookies): pass
+
+    # yes it uses public api but this is only convenient way to call command and get json results
+    @basic_auth
+    @route("POST /api/1/services/{instance_id}/{command_name}")
+    def post_service_command(self, org_id, instance_id, command_name, auth, data="{}"):
+        """
+        :param org_id: not yet used but added for compatibility with future private api
+        """
+        pass
+
+
 class PublicPath(PrivatePath):
 # TODO: Public api hack.
 # We replace private routes with public ones. Fixing response reaction in code.
